@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  *
  *  OBEX Server
@@ -6,20 +7,6 @@
  *  Copyright (C) 2007-2010  Marcel Holtmann <marcel@holtmann.org>
  *
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -42,7 +29,6 @@
 #include "filesystem.h"
 
 #define VCARD_TYPE "text/x-vcard"
-#define VCARD_FILE CONFIGDIR "/vcard.vcf"
 
 static void *opp_connect(struct obex_session *os, int *err)
 {
@@ -62,7 +48,6 @@ static void opp_progress(struct obex_session *os, void *user_data)
 static int opp_chkput(struct obex_session *os, void *user_data)
 {
 	char *folder, *name, *path;
-	int32_t time;
 	const char *t;
 	int err;
 
@@ -79,8 +64,7 @@ static int opp_chkput(struct obex_session *os, void *user_data)
 		goto skip_auth;
 	}
 
-	time = 0;
-	err = manager_request_authorization(user_data, time, &folder, &name);
+	err = manager_request_authorization(user_data, &folder, &name);
 	if (err < 0)
 		return -EPERM;
 
@@ -134,6 +118,8 @@ static int opp_put(struct obex_session *os, void *user_data)
 static int opp_get(struct obex_session *os, void *user_data)
 {
 	const char *type;
+	char *folder, *path;
+	int err = 0;
 
 	if (obex_get_name(os))
 		return -EPERM;
@@ -143,14 +129,19 @@ static int opp_get(struct obex_session *os, void *user_data)
 	if (type == NULL)
 		return -EPERM;
 
+	folder = g_strdup(obex_option_root_folder());
+	path = g_build_filename(folder, "/vcard.vcf", NULL);
+
 	if (g_ascii_strcasecmp(type, VCARD_TYPE) == 0) {
-		if (obex_get_stream_start(os, VCARD_FILE) < 0)
-			return -ENOENT;
+		if (obex_get_stream_start(os, path) < 0)
+			err = -ENOENT;
 
 	} else
-		return -EPERM;
+		err = -EPERM;
 
-	return 0;
+	g_free(folder);
+	g_free(path);
+	return err;
 }
 
 static void opp_disconnect(struct obex_session *os, void *user_data)
