@@ -116,8 +116,8 @@ static void print_le_states(uint64_t states)
 		"Directed Advertising State",
 		"Passive Scanning State",
 		"Active Scanning State",
-		"Initiating State/Connection State in Master Role",
-		"Connection State in the Slave Role",
+		"Initiating State/Connection State in Central Role",
+		"Connection State in the Peripheral Role",
 		"Non-connectable Advertising State and Passive Scanning State combination",
 		"Scannable Advertising State and Passive Scanning State combination",
 		"Connectable Advertising State and Passive Scanning State combination",
@@ -128,17 +128,17 @@ static void print_le_states(uint64_t states)
 		"Directed Advertising State and Active Scanning State combination",
 		"Non-connectable Advertising State and Initiating State combination",
 		"Scannable Advertising State and Initiating State combination",
-		"Non-connectable Advertising State and Master Role combination",
-		"Scannable Advertising State and Master Role combination",
-		"Non-connectable Advertising State and Slave Role combination",
-		"Scannable Advertising State and Slave Role combination",
+		"Non-connectable Advertising State and Central Role combination",
+		"Scannable Advertising State and Central Role combination",
+		"Non-connectable Advertising State and Peripheral Role combination",
+		"Scannable Advertising State and Peripheral Role combination",
 		"Passive Scanning State and Initiating State combination",
 		"Active Scanning State and Initiating State combination",
-		"Passive Scanning State and Master Role combination",
-		"Active Scanning State and Master Role combination",
-		"Passive Scanning State and Slave Role combination",
-		"Active Scanning State and Slave Role combination",
-		"Initiating State and Master Role combination/Master Role and Master Role combination",
+		"Passive Scanning State and Central Role combination",
+		"Active Scanning State and Central Role combination",
+		"Passive Scanning State and Peripheral Role combination",
+		"Active Scanning State and Peripheral Role combination",
+		"Initiating State and Central Role combination/Central Role and Central Role combination",
 		NULL
 	};
 
@@ -1886,50 +1886,92 @@ static struct {
 	char *opt;
 	char *doc;
 } command[] = {
-	{ "up",		cmd_up,		0,		"Open and initialize HCI device" },
-	{ "down",	cmd_down,	0,		"Close HCI device" },
-	{ "reset",	cmd_reset,	0,		"Reset HCI device" },
-	{ "rstat",	cmd_rstat,	0,		"Reset statistic counters" },
-	{ "auth",	cmd_auth,	0,		"Enable Authentication" },
-	{ "noauth",	cmd_auth,	0,		"Disable Authentication" },
-	{ "encrypt",	cmd_encrypt,	0,		"Enable Encryption" },
-	{ "noencrypt",	cmd_encrypt,	0,		"Disable Encryption" },
-	{ "piscan",	cmd_scan,	0,		"Enable Page and Inquiry scan" },
-	{ "noscan",	cmd_scan,	0,		"Disable scan" },
-	{ "iscan",	cmd_scan,	0,		"Enable Inquiry scan" },
-	{ "pscan",	cmd_scan,	0,		"Enable Page scan" },
-	{ "ptype",	cmd_ptype,	"[type]",	"Get/Set default packet type" },
-	{ "lm",		cmd_lm,		"[mode]",	"Get/Set default link mode"   },
-	{ "lp",		cmd_lp,		"[policy]",	"Get/Set default link policy" },
-	{ "name",	cmd_name,	"[name]",	"Get/Set local name" },
-	{ "class",	cmd_class,	"[class]",	"Get/Set class of device" },
-	{ "voice",	cmd_voice,	"[voice]",	"Get/Set voice setting" },
-	{ "iac",	cmd_iac,	"[iac]",	"Get/Set inquiry access code" },
-	{ "inqtpl",	cmd_inq_tpl,	"[level]",	"Get/Set inquiry transmit power level" },
-	{ "inqmode",	cmd_inq_mode,	"[mode]",	"Get/Set inquiry mode" },
-	{ "inqdata",	cmd_inq_data,	"[data]",	"Get/Set inquiry data" },
-	{ "inqtype",	cmd_inq_type,	"[type]",	"Get/Set inquiry scan type" },
-	{ "inqparms",	cmd_inq_parms,	"[win:int]",	"Get/Set inquiry scan window and interval" },
-	{ "pageparms",	cmd_page_parms,	"[win:int]",	"Get/Set page scan window and interval" },
-	{ "pageto",	cmd_page_to,	"[to]",		"Get/Set page timeout" },
-	{ "afhmode",	cmd_afh_mode,	"[mode]",	"Get/Set AFH mode" },
-	{ "sspmode",	cmd_ssp_mode,	"[mode]",	"Get/Set Simple Pairing Mode" },
-	{ "aclmtu",	cmd_aclmtu,	"<mtu:pkt>",	"Set ACL MTU and number of packets" },
-	{ "scomtu",	cmd_scomtu,	"<mtu:pkt>",	"Set SCO MTU and number of packets" },
-	{ "delkey",	cmd_delkey,	"<bdaddr>",	"Delete link key from the device" },
-	{ "oobdata",	cmd_oob_data,	0,		"Get local OOB data" },
-	{ "commands",	cmd_commands,	0,		"Display supported commands" },
-	{ "features",	cmd_features,	0,		"Display device features" },
-	{ "version",	cmd_version,	0,		"Display version information" },
-	{ "revision",	cmd_revision,	0,		"Display revision information" },
-	{ "block",	cmd_block,	"<bdaddr>",	"Add a device to the blacklist" },
-	{ "unblock",	cmd_unblock,	"<bdaddr>",	"Remove a device from the blacklist" },
-	{ "lerandaddr", cmd_le_addr,	"<bdaddr>",	"Set LE Random Address" },
-	{ "leadv",	cmd_le_adv,	"[type]",	"Enable LE advertising"
+	{ "up",		cmd_up,		0,
+		"Open and initialize HCI device" },
+	{ "down",	cmd_down,	0,
+		"Close HCI device" },
+	{ "reset",	cmd_reset,	0,
+		"Reset HCI device" },
+	{ "rstat",	cmd_rstat,	0,
+		"Reset statistic counters" },
+	{ "auth",	cmd_auth,	0,
+		"Enable Authentication" },
+	{ "noauth",	cmd_auth,	0,
+		"Disable Authentication" },
+	{ "encrypt",	cmd_encrypt,	0,
+		"Enable Encryption" },
+	{ "noencrypt",	cmd_encrypt,	0,
+		"Disable Encryption" },
+	{ "piscan",	cmd_scan,	0,
+		"Enable Page and Inquiry scan" },
+	{ "noscan",	cmd_scan,	0,
+		"Disable scan" },
+	{ "iscan",	cmd_scan,	0,
+		"Enable Inquiry scan" },
+	{ "pscan",	cmd_scan,	0,
+		"Enable Page scan" },
+	{ "ptype",	cmd_ptype,	"[type]",
+		"Get/Set default packet type" },
+	{ "lm",		cmd_lm,		"[mode]",
+		"Get/Set default link mode"   },
+	{ "lp",		cmd_lp,		"[policy]",
+		"Get/Set default link policy" },
+	{ "name",	cmd_name,	"[name]",
+		"Get/Set local name" },
+	{ "class",	cmd_class,	"[class]",
+		"Get/Set class of device" },
+	{ "voice",	cmd_voice,	"[voice]",
+		"Get/Set voice setting" },
+	{ "iac",	cmd_iac,	"[iac]",
+		"Get/Set inquiry access code" },
+	{ "inqtpl",	cmd_inq_tpl,	"[level]",
+		"Get/Set inquiry transmit power level" },
+	{ "inqmode",	cmd_inq_mode,	"[mode]",
+		"Get/Set inquiry mode" },
+	{ "inqdata",	cmd_inq_data,	"[data]",
+		"Get/Set inquiry data" },
+	{ "inqtype",	cmd_inq_type,	"[type]",
+		"Get/Set inquiry scan type" },
+	{ "inqparms",	cmd_inq_parms,	"[win:int]",
+		"Get/Set inquiry scan window and interval" },
+	{ "pageparms",	cmd_page_parms,	"[win:int]",
+		"Get/Set page scan window and interval" },
+	{ "pageto",	cmd_page_to,	"[to]",
+		"Get/Set page timeout" },
+	{ "afhmode",	cmd_afh_mode,	"[mode]",
+		"Get/Set AFH mode" },
+	{ "sspmode",	cmd_ssp_mode,	"[mode]",
+		"Get/Set Simple Pairing Mode" },
+	{ "aclmtu",	cmd_aclmtu,	"<mtu:pkt>",
+		"Set ACL MTU and number of packets" },
+	{ "scomtu",	cmd_scomtu,	"<mtu:pkt>",
+		"Set SCO MTU and number of packets" },
+	{ "delkey",	cmd_delkey,	"<bdaddr>",
+		"Delete link key from the device" },
+	{ "oobdata",	cmd_oob_data,	0,
+		"Get local OOB data" },
+	{ "commands",	cmd_commands,	0,
+		"Display supported commands" },
+	{ "features",	cmd_features,	0,
+		"Display device features" },
+	{ "version",	cmd_version,	0,
+		"Display version information" },
+	{ "revision",	cmd_revision,	0,
+		"Display revision information" },
+	{ "block",	cmd_block,	"<bdaddr>",
+		"Add a device to the reject list" },
+	{ "unblock",	cmd_unblock,	"<bdaddr>",
+		"Remove a device from the reject list" },
+	{ "lerandaddr", cmd_le_addr,	"<bdaddr>",
+		"Set LE Random Address" },
+	{ "leadv",	cmd_le_adv,	"[type]",
+		"Enable LE advertising"
 		"\n\t\t\t0 - Connectable undirected advertising (default)"
 		"\n\t\t\t3 - Non connectable undirected advertising"},
-	{ "noleadv",	cmd_no_le_adv,	0,		"Disable LE advertising" },
-	{ "lestates",	cmd_le_states,	0,		"Display the supported LE states" },
+	{ "noleadv",	cmd_no_le_adv,	0,
+			"Disable LE advertising" },
+	{ "lestates",	cmd_le_states,	0,
+			"Display the supported LE states" },
 	{ NULL, NULL, 0 }
 };
 
